@@ -29,7 +29,7 @@
   POSSIBILITY OF SUCH DAMAGE. 
 */
 
-/* $Id: iom32u4.h 2225 2011-03-02 16:27:26Z arcanum $ */
+/* $Id: iom32u4.h 2479 2015-07-22 05:32:39Z pitchumani $ */
 
 /* avr/iom32u4.h - definitions for ATmega32U4. */
 
@@ -172,11 +172,6 @@
 #define OCF1C 3
 #define ICF1 5
 
-#define TIFR2 _SFR_IO8(0x17)
-#define TOV2 0
-#define OCF2A 1
-#define OCF2B 2
-
 #define TIFR3 _SFR_IO8(0x18)
 #define TOV3 0
 #define OCF3A 1
@@ -444,6 +439,8 @@
 #define CLKPS3 3
 #define CLKPCE 7
 
+/* Reserved [0x62..0x63] */
+
 #define PRR0 _SFR_MEM8(0x64)
 #define PRADC 0
 #define PRUSART0 1
@@ -453,11 +450,25 @@
 #define PRTIM2 6
 #define PRTWI 7
 
+#define __AVR_HAVE_PRR0	((1<<PRADC)|(1<<PRUSART0)|(1<<PRSPI)|(1<<PRTIM1)|(1<<PRTIM0)|(1<<PRTIM2)|(1<<PRTWI))
+#define __AVR_HAVE_PRR0_PRADC
+#define __AVR_HAVE_PRR0_PRUSART0
+#define __AVR_HAVE_PRR0_PRSPI
+#define __AVR_HAVE_PRR0_PRTIM1
+#define __AVR_HAVE_PRR0_PRTIM0
+#define __AVR_HAVE_PRR0_PRTIM2
+#define __AVR_HAVE_PRR0_PRTWI
+
 #define PRR1 _SFR_MEM8(0x65)
 #define PRUSART1 0
 #define PRTIM3 3
 #define PRUSB 7
 
+#define __AVR_HAVE_PRR1	((1<<PRUSART1)|(1<<PRTIM3)|(1<<PRUSB))
+#define __AVR_HAVE_PRR1_PRUSART1
+#define __AVR_HAVE_PRR1_PRTIM3
+#define __AVR_HAVE_PRR1_PRUSB
+
 #define OSCCAL _SFR_MEM8(0x66)
 #define CAL0 0
 #define CAL1 1
@@ -504,9 +515,7 @@
 #define PCINT6 6
 #define PCINT7 7
 
-#define PCMSK1 _SFR_MEM8(0x6C)
-
-#define PCMSK2 _SFR_MEM8(0x6D)
+/* Reserved [0x6C..0x6D] */
 
 #define TIMSK0 _SFR_MEM8(0x6E)
 #define TOIE0 0
@@ -520,10 +529,7 @@
 #define OCIE1C 3
 #define ICIE1 5
 
-#define TIMSK2 _SFR_MEM8(0x70)
-#define TOIE2 0
-#define OCIE2A 1
-#define OCIE2B 2
+/* Reserved [0x70] */
 
 #define TIMSK3 _SFR_MEM8(0x71)
 #define TOIE3 0
@@ -538,9 +544,9 @@
 #define OCIE4A 6
 #define OCIE4D 7
 
-#define TIMSK5 _SFR_MEM8(0x73)
+/* Reserved [0x73..0x77] */
 
-#ifndef _ASSEMBLER_
+#ifndef __ASSEMBLER__
 #define ADC _SFR_MEM16(0x78)
 #endif
 #define ADCW _SFR_MEM16(0x78)
@@ -640,6 +646,8 @@
 #define FOC1B 6 
 #define FOC1A 7 
 
+/* Reserved [0x83] */
+
 #define TCNT1 _SFR_MEM16(0x84)
 
 #define TCNT1L _SFR_MEM8(0x84)
@@ -750,6 +758,8 @@
 #define OCR1CH6 6
 #define OCR1CH7 7
 
+/* Reserved [0x8E..0x8F] */
+
 #define TCCR3A _SFR_MEM8(0x90)
 #define WGM30 0
 #define WGM31 1
@@ -774,6 +784,8 @@
 #define FOC3B 6
 #define FOC3A 7
 
+/* Reserved [0x93] */
+
 #define TCNT3 _SFR_MEM16(0x94)
 
 #define TCNT3L _SFR_MEM8(0x94)
@@ -884,89 +896,7 @@
 #define OCR3CH6 6
 #define OCR3CH7 7
 
-#define UHCON _SFR_MEM8(0x9E)
-
-#define UHINT _SFR_MEM8(0x9F)
-
-#define UHIEN _SFR_MEM8(0xA0)
-
-#define UHADDR _SFR_MEM8(0xA1)
-
-#define UHFNUM _SFR_MEM16(0xA2)
-
-#define UHFNUML _SFR_MEM8(0xA2)
-
-#define UHFNUMH _SFR_MEM8(0xA3)
-
-#define UHFLEN _SFR_MEM8(0xA4)
-
-#define UPINRQX _SFR_MEM8(0xA5)
-
-#define UPINTX _SFR_MEM8(0xA6)
-
-#define UPNUM _SFR_MEM8(0xA7)
-
-#define UPRST _SFR_MEM8(0xA8)
-
-#define UPCONX _SFR_MEM8(0xA9)
-
-#define UPCFG0X _SFR_MEM8(0xAA)
-
-#define UPCFG1X _SFR_MEM8(0xAB)
-
-#define UPSTAX _SFR_MEM8(0xAC)
-
-#define UPCFG2X _SFR_MEM8(0xAD)
-
-#define UPIENX _SFR_MEM8(0xAE)
-
-#define UPDATX _SFR_MEM8(0xAF)
-
-#define TCCR2A _SFR_MEM8(0xB0)
-#define WGM20 0
-#define WGM21 1
-#define COM2B0 4
-#define COM2B1 5
-#define COM2A0 6
-#define COM2A1 7
-
-#define TCCR2B _SFR_MEM8(0xB1)
-#define CS20 0
-#define CS21 1
-#define CS22 2
-#define WGM22 3
-#define FOC2B 6
-#define FOC2A 7
-
-#define TCNT2 _SFR_MEM8(0xB2)
-#define TCNT2_0 0
-#define TCNT2_1 1
-#define TCNT2_2 2
-#define TCNT2_3 3
-#define TCNT2_4 4
-#define TCNT2_5 5
-#define TCNT2_6 6
-#define TCNT2_7 7
-
-#define OCR2A _SFR_MEM8(0xB3)
-#define OCR2_0 0
-#define OCR2_1 1
-#define OCR2_2 2
-#define OCR2_3 3
-#define OCR2_4 4
-#define OCR2_5 5
-#define OCR2_6 6
-#define OCR2_7 7
-
-#define OCR2B _SFR_MEM8(0xB4)
-#define OCR2_0 0
-#define OCR2_1 1
-#define OCR2_2 2
-#define OCR2_3 3
-#define OCR2_4 4
-#define OCR2_5 5
-#define OCR2_6 6
-#define OCR2_7 7
+/* Reserved [0x9E..0xB7] */
 
 #define TWBR _SFR_MEM8(0xB8)
 #define TWBR0 0
@@ -1146,6 +1076,10 @@
 #define UMSEL10 6
 #define UMSEL11 7
 
+#define UCSR1D _SFR_MEM8(0xCB)
+#define RTSEN 0
+#define CTSEN 1
+
 #define UBRR1 _SFR_MEM16(0xCC)
 
 #define UBRR1L _SFR_MEM8(0xCC)
@@ -1202,6 +1136,8 @@
 #define OCR4D6 6
 #define OCR4D7 7
 
+/* Reserved [0xD3] */
+
 #define DT4 _SFR_MEM8(0xD4)
 #define DT4L0 0
 #define DT4L1 1
@@ -1212,27 +1148,25 @@
 #define DT4L6 6
 #define DT4L7 7
 
-#define UHWCON _SFR_MEM8(0xD7)
-#define UVREGE 0
+/* Reserved [0xD5..0xD6] */
 
-#define USBCON _SFR_MEM8(0xD8)
-#define VBUSTE 0
+#define UHWCON  _SFR_MEM8(0xD7)
+#define UVREGE  0
+
+#define USBCON  _SFR_MEM8(0xD8)
+#define VBUSTE  0
 #define OTGPADE 4
-#define FRZCLK 5
-#define USBE 7
+#define FRZCLK  5
+#define USBE    7
 
-#define USBSTA _SFR_MEM8(0xD9)
-#define VBUS 0
-#define SPEED 3
+#define USBSTA  _SFR_MEM8(0xD9)
+#define VBUS    0
+#define SPEED   3
 
-#define USBINT _SFR_MEM8(0xDA)
-#define VBUSTI 0
+#define USBINT  _SFR_MEM8(0xDA)
+#define VBUSTI  0
 
-#define OTGCON _SFR_MEM8(0xDD)
-
-#define OTGIEN _SFR_MEM8(0xDE)
-
-#define OTGINT _SFR_MEM8(0xDF)
+/* Reserved [0xDB..0xDF] */
 
 #define UDCON _SFR_MEM8(0xE0)
 #define DETACH 0
@@ -1286,7 +1220,7 @@
 #define UDMFN _SFR_MEM8(0xE6)
 #define FNCERR 4
 
-#define UDTST _SFR_MEM8(0xE7)
+/* Reserved [0xE7] */
 
 #define UEINTX _SFR_MEM8(0xE8)
 #define TXINI 0
@@ -1387,15 +1321,6 @@
 #define EPINT5 5
 #define EPINT6 6
 
-#define UPERRX _SFR_MEM8(0xF5)
-
-#define UPBCLX _SFR_MEM8(0xF6)
-
-#define UPBCHX _SFR_MEM8(0xF7)
-
-#define UPINT _SFR_MEM8(0xF8)
-
-#define OTGTCON _SFR_MEM8(0xF9)
 
 
 
@@ -1576,4 +1501,12 @@
 
 
 
+#define SLEEP_MODE_IDLE (0x00<<1)
+#define SLEEP_MODE_ADC (0x01<<1)
+#define SLEEP_MODE_PWR_DOWN (0x02<<1)
+#define SLEEP_MODE_PWR_SAVE (0x03<<1)
+#define SLEEP_MODE_STANDBY (0x06<<1)
+#define SLEEP_MODE_EXT_STANDBY (0x07<<1)
+
+
 #endif  /* _AVR_IOM32U4_H_ */
